#!/usr/local/cpanel/3rdparty/bin/perl

use strict;
use warnings;

use Data::Dumper;
use Getopt::Std;
use File::Copy;

my %opts;
my ($tokfile, $oldfile);
my (%hashfile, %oldhashfile);
my @a_tok;
my $token;
my $domains;
my @first;
my @domains_to_check;
my $copyfile = 0;
my $verbose = 0;
my $quiet = 0;
my $config = "/etc/Raptor-FAST/config";
my $systatus;

# XXX read from a config file ?
my $virtusertable ||= '/home/*/etc/*';
my @dir = glob $virtusertable;
my @pathdom;

=head1 NAME

raptor_fast_cpanel_cron - Tool to send email users data from cPanel to Raptor FAST via cron

=head1 SYNOPSIS

B<raptor_fast_cpanel>

=head1 DESCRIPTION

Tool to send email users data from cPanel to Raptor FAST, the software is scheduled via crontab(5) and
it monitors C<config.token> file for changes.

The software reads parameters via getopt(1) and accepts the file to monitor (-f) and the program
to exec (-x) for each domain that has changed.

=cut

getopts('c:f:qx:v', \%opts);

if(defined $opts{'s'}) {
  $quiet = 1;
}

if(defined $opts{'c'}) {
  $config = $opts{'c'};
}

if((not defined $opts{'f'})) {
  warn "Please specify a config.token file to monitor\n";
  exit;
} else {
  $oldfile = $opts{'f'}. ".old";
}

if((not defined $opts{'x'})) {
  if(not $quiet) {
    warn "Please specify a raptor_fast_cpanel.pl file to exec if config.token file hash changed\n";
  }
  exit;
}

if(defined $opts{'v'}) {
  $verbose = 1;
}

# if there is no old file, create a new one
if ( ! -f $oldfile ) {
  open(FILE, ">" . $oldfile) || die("cannot open file: " . $!);
  close(FILE);
}

open $tokfile, $opts{'f'} or die "Could not open $opts{'f'}: $!";
while( my $line = <$tokfile>) {
  next if ( $line =~ /^#/ );
  @a_tok = split(/:/, $line);
  $token = $a_tok[0];
  $domains = $a_tok[1];
  chomp($domains);
  $hashfile{$token} = $domains;
}
close $tokfile;

open $tokfile, $oldfile or die "Could not open $oldfile: $!";
while( my $line = <$tokfile>) {
  next if ( $line =~ /^#/ );
  @a_tok = split(/:/, $line);
  $token = $a_tok[0];
  $domains = $a_tok[1];
  chomp($domains);
  $oldhashfile{$token} = $domains;
}
close $tokfile;

while(my($k, $v) = each %hashfile) {
  if(exists $oldhashfile{$k}) {
    @first = split(/,/, $hashfile{$k});
    my @second = split(/,/, $oldhashfile{$k});
    my %second = map {$_=>1} @second;
    my @only_in_first = grep { !$second{$_} } @first;
    push(@domains_to_check, @only_in_first);
  } else {
    @first = split(/,/, $hashfile{$k});
    push(@domains_to_check, @first);
  }
}

if((defined $domains_to_check[0]) and ($domains_to_check[0] eq "ALL")) {
  if($verbose eq 1) {
    $systatus = system($opts{'x'} . " -v -d $domains_to_check[0]");
  } else {
    $systatus = system($opts{'x'} . " -d $domains_to_check[0]");
  }
  if($systatus eq 0) {
    $copyfile = 1;
  } else {
    $copyfile = 0;
    exit;
  }
} else {
  foreach my $dom ( @domains_to_check ) {
    if($verbose eq 1) {
      $systatus = system($opts{'x'} . " -v -d $dom");
    } else {
      $systatus = system($opts{'x'} . " -d $dom");
    }
    if($systatus eq 0) {
      $copyfile = 1;
    } else {
      $copyfile = 0;
      exit;
    }
  }
}
if($copyfile eq 1) {
  copy($opts{'f'}, $oldfile) or die "Copy failed: $!";
}
